#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
import sys
import os.path
import argparse
from string import Template

dirname = os.path.dirname(__file__)
imported_files = os.path.join(dirname, "../src/")
sys.path.insert(0, os.path.abspath(imported_files))

from zeo_connector_defaults import _SERVER_CONF_PATH
from zeo_connector_defaults import _CLIENT_CONF_PATH


# Functions & classes =========================================================
def create_configuration(args):
    with open(_SERVER_CONF_PATH) as f:
        server_template = f.read()

    with open(_CLIENT_CONF_PATH) as f:
        client_template = f.read()

    if not args.only_server:
        client = Template(client_template).substitute(
            server=args.server,
            port=args.port,
        )
        with open(os.path.basename(_CLIENT_CONF_PATH), "w") as f:
            f.write(client)

    if not args.only_client:
        server = Template(server_template).substitute(
            server=args.server,
            port=args.port,
            path=args.path,
        )
        with open(os.path.basename(_SERVER_CONF_PATH), "w") as f:
            f.write(server)


# Main program ================================================================
if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description="""This program will create the default ZEO XML
            configuration files."""
    )
    parser.add_argument(
        "-s",
        "--server",
        default="localhost",
        help="Server url. Default: localhost"
    )
    parser.add_argument(
        "-p",
        "--port",
        default=60985,
        type=int,
        help="Port of the server. Default: 60985"
    )
    parser.add_argument(
        "path",
        metavar="PATH",
        nargs='?',
        default="",
        help="""Path to the database on the server (used in server
            configuration only."""
    )
    parser.add_argument(
        "-C",
        "--only-client",
        action="store_true",
        help="Create only CLIENT configuration."
    )
    parser.add_argument(
        "-S",
        "--only-server",
        action="store_true",
        help="Create only SERVER configuration."
    )

    args = parser.parse_args()

    if not args.only_client and not args.path:
        sys.stderr.write(
            "You have to specify path to the database on for the DB server.\n"
        )
        sys.exit(1)

    if args.only_client and args.only_server:
        sys.stderr.write(
            "You can't have --only-client and --only-server together!\n"
        )
        sys.exit(1)

    create_configuration(args)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
import shutil
import os.path
import tempfile
import threading
import subprocess
from string import Template


# Variables ===================================================================
SERV = None
TMP_DIR = None


# Functions & classes =========================================================
def data_context_name(fn):
    return os.path.join(os.path.dirname(__file__), "test_data", fn)


def data_context(fn, mode="r"):
    with open(data_context_name(fn), mode) as f:
        return f.read()


def tmp_context_name(fn=None):
    if not fn:
        return TMP_DIR

    return os.path.join(TMP_DIR, fn)


def tmp_context(fn, mode="r"):
    with open(tmp_context_name(fn), mode) as f:
        return f.read()


# Environment generators ======================================================
def generate_environment():
    global TMP_DIR
    TMP_DIR = tempfile.mkdtemp()

    # write ZEO server config to  temp directory
    zeo_conf_path = os.path.join(TMP_DIR, "zeo.conf")
    with open(zeo_conf_path, "w") as f:
        f.write(
            Template(data_context("zeo.conf")).substitute(path=TMP_DIR)
        )

    # write client config to temp directory
    client_config_path = os.path.join(TMP_DIR, "zeo_client.conf")
    shutil.copy(data_context_name("zeo_client.conf"), client_config_path)

    # run the ZEO server
    def run_zeo():
        global SERV
        SERV = subprocess.Popen(["runzeo", "-C", zeo_conf_path])

    serv = threading.Thread(target=run_zeo)
    serv.setDaemon(True)
    serv.start()


def cleanup_environment():
    SERV.terminate()
    shutil.rmtree(TMP_DIR)
    if os.path.exists(TMP_DIR):
        os.rmdir(TMP_DIR)

    global TMP_DIR
    TMP_DIR = None



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
import os.path

from environment_generator import tmp_context
from environment_generator import tmp_context_name

from environment_generator import generate_environment
from environment_generator import cleanup_environment


# Functions ===================================================================
def _in_path(fn, dirname="default_data"):
    pwd = os.path.dirname(__file__)
    return os.path.join(os.path.abspath(pwd), dirname, fn)


# Variables ===================================================================
SERVER_CONF_PATH = _in_path(fn="zeo.conf", dirname="default_data")
CLIENT_CONF_PATH = _in_path(fn="zeo_client.conf", dirname="default_data")

_SERVER_CONF_PATH = _in_path(fn="zeo.conf", dirname="template_data")
_CLIENT_CONF_PATH = _in_path(fn="zeo_client.conf", dirname="template_data")



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from setuptools import setup
from setuptools import find_packages


# Variables ===================================================================
changelog = open('CHANGELOG.rst').read()
long_description = "\n\n".join([
    open('README.rst').read(),
    changelog
])


# Functions ===================================================================
def allSame(s):
    return not any(filter(lambda x: x != s[0], s))


def hasDigit(s):
    return any(char.isdigit() for char in s)


def getVersion(data):
    """
    Parse version from changelog written in RST format.
    """
    data = data.splitlines()
    return next((
        v
        for v, u in zip(data, data[1:])  # v = version, u = underline
        if len(v) == len(u) and allSame(u) and hasDigit(v) and "." in v
    ))


# Actual setup definition =====================================================
setup(
    name='zeo_connector_defaults',
    version=getVersion(changelog),
    description="Default conf. files and conf. file generator for `zeo_connector`.",
    long_description=long_description,
    url='https://github.com/Bystroushaak/zeo_connector_defaults',

    author='Bystroushaak',
    author_email='bystrousak@kitakitsune.org',

    classifiers=[
        "Development Status :: 4 - Beta",
        'Intended Audience :: Developers',

        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",

        "License :: OSI Approved :: MIT License",
    ],
    license='MIT',

    packages=find_packages('src'),
    package_dir={'': 'src'},

    scripts=[
        'bin/zeo_connector_gen_defaults.py',
    ],

    zip_safe=False,
    include_package_data=True,
)
